@@ -402,6 +402,15 @@ test_gvalue_ret_callback (GObject *object, GType type)
   return ret;
 }
 
+static GParamSpec *
+test_paramspec_in_callback (GObject *object, GParamSpec *p)
+{
+  g_return_val_if_fail (G_IS_OBJECT (object), NULL);
+  g_return_val_if_fail (G_IS_PARAM_SPEC (p), NULL);
+
+  return p;
+}
+
 static void
 connectcallbacks (GObject *object)
 {
@@ -460,6 +469,10 @@ connectcallbacks (GObject *object)
                     "test_gvalue_ret",
                     G_CALLBACK (test_gvalue_ret_callback), 
                     NULL);
+  g_signal_connect (G_OBJECT (object),
+                    "test_paramspec_in",
+                    G_CALLBACK (test_paramspec_in_callback), 
+                    NULL);
 }
 
 static PyObject *
@@ -530,7 +543,6 @@ _wrap_test_gerror_exception(PyObject *self, PyObject *args)
         return NULL;
     }	    
 
-    Py_DECREF(py_method);
     Py_DECREF(py_args);
     Py_DECREF(py_ret);
 
